/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef EB_FONT_H
#define EB_FONT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>

#ifdef EB_BUILD_LIBRARY
#include "types.h"
#else
#include <eb/types.h>
#endif


/*
 * Font types.
 */
#define EB_FONT_16		16
#define EB_FONT_24		24
#define EB_FONT_30		30
#define EB_FONT_48		48

/*
 * Font sizes.
 */
#define EB_SIZE_NARROW_FONT_16	16
#define EB_SIZE_WIDE_FONT_16	32
#define EB_SIZE_NARROW_FONT_24	48
#define EB_SIZE_WIDE_FONT_24	72
#define EB_SIZE_NARROW_FONT_30	60
#define EB_SIZE_WIDE_FONT_30	120
#define EB_SIZE_NARROW_FONT_48	144
#define EB_SIZE_WIDE_FONT_48	288

/*
 * Font width.
 */
#define EB_WIDTH_NARROW_FONT_16	8
#define EB_WIDTH_WIDE_FONT_16	16
#define EB_WIDTH_NARROW_FONT_24	16
#define EB_WIDTH_WIDE_FONT_24	24
#define EB_WIDTH_NARROW_FONT_30	16
#define EB_WIDTH_WIDE_FONT_30	32
#define EB_WIDTH_NARROW_FONT_48	24
#define EB_WIDTH_WIDE_FONT_48	48

/*
 * Bitmap image sizes.
 */
#define EB_SIZE_NARROW_FONT_16_XBM		184
#define EB_SIZE_WIDE_FONT_16_XBM		284
#define EB_SIZE_NARROW_FONT_16_XPM		266
#define EB_SIZE_WIDE_FONT_16_XPM		395
#define EB_SIZE_NARROW_FONT_16_GIF		186
#define EB_SIZE_WIDE_FONT_16_GIF		314

#define EB_SIZE_NARROW_FONT_24_XBM		383
#define EB_SIZE_WIDE_FONT_24_XBM		533
#define EB_SIZE_NARROW_FONT_24_XPM		555
#define EB_SIZE_WIDE_FONT_24_XPM		747
#define EB_SIZE_NARROW_FONT_24_GIF		450
#define EB_SIZE_WIDE_FONT_24_GIF		642

#define EB_SIZE_NARROW_FONT_30_XBM		458
#define EB_SIZE_WIDE_FONT_30_XBM		833
#define EB_SIZE_NARROW_FONT_30_XPM		675
#define EB_SIZE_WIDE_FONT_30_XPM		1155
#define EB_SIZE_NARROW_FONT_30_GIF		552
#define EB_SIZE_WIDE_FONT_30_GIF		1032

#define EB_SIZE_NARROW_FONT_48_XBM		983
#define EB_SIZE_WIDE_FONT_48_XBM		1883
#define EB_SIZE_NARROW_FONT_48_XPM		1419
#define EB_SIZE_WIDE_FONT_48_XPM		2571
#define EB_SIZE_NARROW_FONT_48_GIF		1242
#define EB_SIZE_WIDE_FONT_48_GIF		2394

#define EB_SIZE_FONT_IMAGE	EB_SIZE_WIDE_FONT_48_XPM	    

/*
 * Function declarations.
 */
#ifdef __STDC__

/* bitmap.c */
int eb_narrow_font_xbm_size(EB_Font_Code);
int eb_narrow_font_xpm_size(EB_Font_Code);
int eb_narrow_font_gif_size(EB_Font_Code);
int eb_wide_font_xbm_size(EB_Font_Code);
int eb_wide_font_xpm_size(EB_Font_Code);
int eb_wide_font_gif_size(EB_Font_Code);
size_t eb_convert_bitmap_xbm(char *, const char *, int, int);
size_t eb_convert_bitmap_xpm(char *, const char *, int, int);
size_t eb_convert_bitmap_gif(char *, const char *, int, int);
/* font.c */
EB_Font_Code eb_font(EB_Book *);
int eb_set_font(EB_Book *, EB_Font_Code);
void eb_unset_font(EB_Book *);
int eb_font_count(EB_Book *);
int eb_font_list(EB_Book *, EB_Font_Code *);
int eb_have_font(EB_Book *, EB_Font_Code);
/* narwfont.c */
int eb_have_narrow_font(EB_Book *);
int eb_narrow_font_width(EB_Book *);
int eb_narrow_font_width2(EB_Font_Code);
int eb_narrow_font_size(EB_Book *);
int eb_narrow_font_size2(EB_Font_Code);
const char *eb_narrow_font_filename(EB_Book *);
const char *eb_narrow_font_filename2(EB_Book *, EB_Font_Code);
int eb_narrow_font_start(EB_Book *);
int eb_narrow_font_end(EB_Book *);
int eb_narrow_character_bitmap(EB_Book *, int, char *);
int eb_forward_narrow_character(EB_Book *, int, int);
int eb_backward_narrow_character(EB_Book *, int, int);
/* widefont.c */
int eb_have_wide_font(EB_Book *);
int eb_wide_font_width(EB_Book *);
int eb_wide_font_width2(EB_Font_Code);
int eb_wide_font_size(EB_Book *);
int eb_wide_font_size2(EB_Font_Code);
const char *eb_wide_font_filename(EB_Book *);
const char *eb_wide_font_filename2(EB_Book *, EB_Font_Code);
int eb_wide_font_start(EB_Book *);
int eb_wide_font_end(EB_Book *);
int eb_wide_character_bitmap(EB_Book *, int, char *);
int eb_forward_wide_character(EB_Book *, int, int);
int eb_backward_wide_character(EB_Book *, int, int);

#else /* not __STDC__ */

/* image.c */
int eb_narrow_font_xbm_size();
int eb_narrow_font_xpm_size();
int eb_narrow_font_gif_size();
int eb_wide_font_xbm_size();
int eb_wide_font_xpm_size();
int eb_wide_font_gif_size();
size_t eb_convert_bitmap_xbm();
size_t eb_convert_bitmap_xpm();
size_t eb_convert_bitmap_gif();
/* font.c */
EB_Font_Code eb_font();
int eb_set_font();
void eb_unset_font();
int eb_font_count();
int eb_font_list();
int eb_have_font();
/* narwfont.c */
int eb_have_narrow_font();
int eb_narrow_font_width();
int eb_narrow_font_width2();
int eb_narrow_font_size();
int eb_narrow_font_size2();
const char *eb_narrow_font_filename();
const char *eb_narrow_font_filename2();
int eb_narrow_font_start();
int eb_narrow_font_end();
int eb_narrow_character_bitmap();
int eb_next_narrow_character()
int eb_previous_narrow_character();
/* widefont.c */
int eb_have_wide_font();
int eb_wide_font_width();
int eb_wide_font_width2();
int eb_wide_font_size();
int eb_wide_font_size2();
const char *eb_wide_font_filename();
const char *eb_wide_font_filename2();
int eb_wide_font_start();
int eb_wide_font_end();
int eb_wide_character_bitmap();
int eb_forward_wide_character();
int eb_backward_wide_character();

#endif /* not __STDC__ */


#ifdef __cplusplus
}
#endif

#endif /* not EB_FONT_H */
