/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
#include <string.h>
#if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
#include <memory.h>
#endif /* not STDC_HEADERS and HAVE_MEMORY_H */
#else /* not STDC_HEADERS and not HAVE_STRING_H */
#include <strings.h>
#endif /* not STDC_HEADERS and not HAVE_STRING_H */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "permission.h"


/*
 * Initialize `permission'.
 */
void
initialize_permission(permission)
    Permission *permission;
{
    permission->next = NULL;
}


/*
 * Free all nodes in `permission'.
 */
void
clear_permission(permission)
    Permission *permission;
{
    Permission *pm, *next;

    pm = permission->next;
    while (pm != NULL) {
	next = (Permission *)pm->next;
	free(pm->host);
	free(pm);
	pm = next;
    }

    permission->next = NULL;
}


/*
 * Add `hostnme' to the list `permission'.
 * If succeeded, 0 is returned.  Otherwise, -1 is returned.
 */
int
add_permission(permission, hostname)
    Permission *permission;
    const char *hostname;
{
    Permission *pm = NULL;

    /*
     * Allocate memeories for new node and a hostname.
     */
    pm = (Permission *)malloc(sizeof(Permission));
    if (pm == NULL)
	goto failed;
    pm->host = NULL;

    if (hostname != NULL) {
	pm->host = (char *)malloc(strlen(hostname) + 1);
	if (pm->host == NULL)
	    goto failed;
    }

    /*
     * Insert the new node into the list `perm'.
     */
    strcpy(pm->host, hostname);
    pm->next = permission->next;
    permission->next = pm;

    return 0;

    /*
     * If an error occurs...
     */
  failed:
    if (pm != NULL) {
	if (pm->host != NULL)
	    free(pm->host);
	free(pm);
    }
    return -1;
}


/*
 * Examine access permission to `hostname' and `address'.
 * If access is granted, 1 is returned.  Otherwise 0 is returned.
 */
int
test_permission(permission, hostname, address, func)
    const Permission *permission;
    const char *hostname;
    const char *address;
#ifdef __STDC__
    int (*func)(const char *, const char *);
#else
    int (*func)();
#endif
{
    const Permission *pm;
    int flag;

    for (pm = permission->next, flag = 0;
	 pm != NULL; pm = (Permission *)pm->next) {

	if (*(pm->host) == '!') {
	    if (func(pm->host + 1, hostname))
		flag = 0;
	    else if (func(pm->host + 1, address))
		flag = 0;
	} else {
	    if (func(pm->host, hostname))
		flag = 1;
	    else if (func(pm->host, address))
		flag = 1;
	}
    }

    return flag;
}


